==> MuSE-0.6.5.tar.gz: MuSE-0.6.5/playlist.cpp <==
/* MuSE - Multiple Streaming Engine
 * Copyright (C) 2000-2002 Denis Roio aka jaromil <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <jutils.h>
#include <playlist.h>
#include <generic.h>

Entry::Entry(char *file) {
  name = strdup(file);
}

Entry::~Entry() {
  free(name);
}

Playlist::Playlist() {
  length = 0;
  first = NULL;
  selected = NULL;
  last = NULL;
}

Playlist::~Playlist() {
  clear();
}

void Playlist::add(char *file) {
  Entry *ptr;

  if(first==NULL) { /* that's the first entry */
    first = new Entry(file);
    first->next = NULL;
    first->prev = NULL;
    last = first;
  } else { /* add the entry to the end */
    ptr = first;
    while(ptr->next!=NULL) ptr = ptr->next;
    ptr->next = new Entry(file);
    ptr->next->next = NULL;
    ptr->next->prev = ptr;
    last = ptr->next;
  }
  length++;
}
  
void Playlist::clear() {
  if(length>0) {
    Entry *ptr = first, *tptr;
    int c;
    free(filename);
    for(c=0;c<length;c++) {
      tptr = ptr->next;
      delete ptr;
      ptr = tptr;
    }
    length = 0;
    first = NULL;
    selected = NULL;
    last = NULL;
  }
      
}

bool Playlist::load(char *file) {
  func("Playlist::load(%s)",file);
  char temp[MAX_PATH_SIZE], *p;
  int c = 0;
  FILE *fd;

  clear();

  fd = fopen(file, "r");
  if(fd==NULL)
    return(false);
  while(fgets(temp,MAX_PATH_SIZE,fd)!=NULL) {
    for(p=temp;*p!='\n';p++);

    if(*p=='\n') *p = '\0';
    else {
      error("EHY! i got an invalid filename here");
      act("try to substitute null-spaces and be sure it's not longer than %u chars",MAX_PATH_SIZE);
      return(false);
    }
    add(temp);
    c++;
  }
  fclose(fd);
  if(c>0) {
    filename = strdup(file);
    length = c;
  }
  return(true);
}

char *Playlist::song(int pos) {
  if((length<pos)||(pos<=0)) return(NULL);
  if(pos==1) return(first->name);
  if(pos==length) return(last->name);
  
  Entry *ptr = first;
  for(int c=1;c<pos;c++) {
    if(ptr->next == NULL)
      return(NULL);
    ptr = ptr->next;
  }
  return(ptr->name);
}

void Playlist::select(int pos) {
  if((length<pos)||(pos<=0)) selected = NULL;
  if(pos==1) selected = first;
  if(pos==length) selected = last;

  Entry *ptr = first;
  int c;
  
  for(c=1;c<pos;c++) {
    if(ptr->next == NULL)
      return;
    ptr = ptr->next;
  }
  selected = ptr;
}

void Playlist::select(Entry *sel) {
  if(!sel) return;
  selected = sel;
}
  

Entry *Playlist::pick(int pos) {
  if((length<pos)||(pos==0)) return(NULL);
  if(pos==1) return(first);
  if(pos==length) return(last);

  Entry *ptr = first;
  int c;
  for(c=1;c<pos;c++) ptr = ptr->next;

  return(ptr);
}

bool Playlist::moveup(int pos) {
  if(pos<=1) return(false);
  Entry *p = pick(pos);
  Entry *prev = p->prev, *next = p->next, *pp = prev->prev;
  
  if(next!=NULL) next->prev = prev;
  p->next = prev;
  p->prev = pp;
  prev->next = next;
  prev->prev = p;
  if(pp!=NULL) pp->next = p;
  
  if(pos==2) first = p;
  if(pos==length) last = prev;
  return(true);
}

bool Playlist::movedown(int pos) {
  if(pos>=length) return(false);
  Entry *p = pick(pos);
  Entry *prev = p->prev, *next = p->next, *nn = next->next;

  if(prev!=NULL) prev->next = next;
  p->prev = next;
  p->next = nn;
  next->prev = prev;
  next->next = p;
  if(nn!=NULL) nn->prev = p;
  
  if(pos==length-1) last = p;
  if(pos==1) first = next;
  return(true);
}

==> MuSE-0.7pre1.tar.gz: MuSE-0.7pre1/playlist.cpp <==
/* MuSE - Multiple Streaming Engine
 * Copyright (C) 2000-2002 Denis Roio aka jaromil <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <jutils.h>
#include <playlist.h>
#include <generic.h>
#include <config.h>

Entry::Entry(char *file) {
  name = strdup(file);
}

Entry::~Entry() {
  /* free(name); */
}

Playlist::Playlist() {
  length = 0;
  filename = NULL;
  first = NULL;
  selected = NULL;
  last = NULL;
}

Playlist::~Playlist() {
  clear();
}

void Playlist::add(char *file) {
  func("Playlist::add(%s)",file);
  Entry *ptr;

  if(first==NULL) { /* that's the first entry */
    first = new Entry(file);
    first->next = NULL;
    first->prev = NULL;
    last = first;
  } else { /* add the entry to the end */
    ptr = first;
    while(ptr->next!=NULL) ptr = ptr->next;
    ptr->next = new Entry(file);
    ptr->next->next = NULL;
    ptr->next->prev = ptr;
    last = ptr->next;
  }
  length++;
}

void Playlist::rem(int pos) {
  Entry *ptr = pick(pos);
  if(ptr==NULL) return;
  Entry *prev = ptr->prev, *next = ptr->next;

  /* we are on it, now take it out */
  if(prev!=NULL)
    prev->next = next;
  else first = next;
  
  if(next!=NULL)
    next->prev = prev;
  else last = prev;
  
  delete(ptr);
  
  length--;
}
  
void Playlist::clear() {
  if(length>0) {
    Entry *ptr = first, *tptr;
    int c;
    if(filename) free(filename);
    for(c=0;c<length;c++) {
      tptr = ptr->next;
      delete ptr;
      ptr = tptr;
    }
    length = 0;
    first = NULL;
    selected = NULL;
    last = NULL;
  }
}

bool Playlist::load(char *file) {
  func("Playlist::load(%s)",file);
  char temp[MAX_PATH_SIZE], *p;
  int c = 0;
  FILE *fd;

  clear();

  fd = fopen(file, "r");
  if(fd==NULL)
    return(false);
  while(fgets(temp,MAX_PATH_SIZE,fd)!=NULL) {
    for(p=temp;*p!='\n';p++);

    if(*p=='\n') *p = '\0';
    else {
      error("EHY! i got an invalid filename here");
      act("try to substitute null-spaces and be sure it's not longer than %u chars",MAX_PATH_SIZE);
      return(false);
    }
    add(temp);
    c++;
  }
  fclose(fd);
  if(c>0) {
    filename = strdup(file);
    length = c;
  }
  return(true);
}

char *Playlist::song(int pos) {
  if((length<pos)||(pos<=0)) return(NULL);
  if(pos==1) return(first->name);
  if(pos==length) return(last->name);
  
  Entry *ptr = first;
  for(int c=1;c<pos;c++) {
    if(ptr->next == NULL)
      return(NULL);
    ptr = ptr->next;
  }
  return(ptr->name);
}

void Playlist::sel(int pos) {
  if((length<pos)||(pos<=0)) selected = NULL;
  if(pos==1) selected = first;
  if(pos==length) selected = last;

  Entry *ptr = first;
  int c;
  
  for(c=1;c<pos;c++) {
    if(ptr->next == NULL)
      return;
    ptr = ptr->next;
  }
  selected = ptr;
}

void Playlist::sel(Entry *sel) {
  if(!sel) return;
  selected = sel;
}

int Playlist::pos(Entry *sel) {
  if(!sel) return -1;
  Entry *ptr = first;
  int c;
  for(c=1;c<length;c++) {
    if(ptr == sel) return c;
    ptr = ptr->next;
  }
  return -1;
}
    
Entry *Playlist::pick(int pos) {
  if((length<pos)||(pos==0)) return(NULL);
  if(pos==1) return(first);
  if(pos==length) return(last);

  Entry *ptr = first;
  int c;
  for(c=1;c<pos;c++) ptr = ptr->next;

  return(ptr);
}

bool Playlist::moveup(int pos) {
  if(pos<=1) return(false);
  Entry *p = pick(pos);
  Entry *prev = p->prev, *next = p->next, *pp = prev->prev;
  
  if(next!=NULL) next->prev = prev;
  p->next = prev;
  p->prev = pp;
  prev->next = next;
  prev->prev = p;
  if(pp!=NULL) pp->next = p;
  
  if(pos==2) first = p;
  if(pos==length) last = prev;
  return(true);
}

bool Playlist::movedown(int pos) {
  if(pos>=length) return(false);
  Entry *p = pick(pos);
  Entry *prev = p->prev, *next = p->next, *nn = next->next;

  if(prev!=NULL) prev->next = next;
  p->prev = next;
  p->next = nn;
  next->prev = prev;
  next->next = p;
  if(nn!=NULL) nn->prev = p;
  
  if(pos==length-1) last = p;
  if(pos==1) first = next;
  return(true);
}
